@@ -8,8 +8,8 @@ import (
 	"os"
 	"testing"
 
-	"golang.org/x/tools/go/packages/packagestest"
 	"golang.org/x/tools/gopls/internal/hooks"
+	"golang.org/x/tools/internal/lsp/bug"
 	cmdtest "golang.org/x/tools/internal/lsp/cmd/test"
 	"golang.org/x/tools/internal/lsp/source"
 	"golang.org/x/tools/internal/lsp/tests"
@@ -17,17 +17,13 @@ import (
 )
 
 func TestMain(m *testing.M) {
+	bug.PanicOnBugs = true
 	testenv.ExitIfSmallMachine()
 	os.Exit(m.Run())
 }
 
 func TestCommandLine(t *testing.T) {
-	packagestest.TestAll(t,
-		cmdtest.TestCommandLine(
-			"../../internal/lsp/testdata",
-			commandLineOptions,
-		),
-	)
+	cmdtest.TestCommandLine(t, "../../internal/lsp/testdata", commandLineOptions)
 }
 
 func commandLineOptions(options *source.Options) {
